# Generated by Django 2.2.12 on 2021-06-16 08:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moneta', '0003_auto_20210530_1400'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='status',
            field=models.CharField(choices=[('CHECK', 'CHECK'), ('PAID', 'PAID')], default='CHECK', max_length=6),
        ),
        migrations.AlterField(
            model_name='paymentsystem',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from .config import (
BasicConfig, MainConfig, MainSettings, UrlsConfig, moneta_config
)

from .url_generator import (UrlGenerator, QueryPayload)

from .soap_client import SoapClient

__all__ = [
    "BasicConfig",
    "MainConfig",
    "MainSettings",
    "UrlsConfig",
    "UrlGenerator",
    "QueryPayload",
    "moneta_config",
    "SoapClient",
]





from urllib.parse import urlencode, urlparse, parse_qs


def add_url_query_params(url: str, additional_params: dict) -> str:
    url_components = urlparse(url)
    original_params = parse_qs(url_components.query)
    merged_params = {**original_params, **additional_params}
    updated_query = urlencode(merged_params, doseq=True)
    return url_components._replace(query=updated_query).geturl()



import datetime
from typing import List, OrderedDict, Optional
from pydash import py_

import zeep
from zeep.wsse.username import UsernameToken

import logging


class Pager:
    pageSize: int
    pageNumber: int


class SoapClient:
    def __init__(self, url: str, login: str, password: str):
        token = UsernameToken(login, password)
        self.client = zeep.Client(url, wsse=token)

    @staticmethod
    def debug_mode():
        logging.basicConfig(level=logging.DEBUG)
        logging.getLogger("suds.client").setLevel(logging.DEBUG)

    @staticmethod
    def _convert_to_list_of_dicts(operations: List[OrderedDict]):
        results = []
        for item in operations:
            elm = dict()
            elm["id"] = item.get("id")
            for d in item.get("attribute"):
                elm[d["key"]] = (
                    d["value"] if d["key"] != "sourceamount" else int(d["value"])
                )
            results.append(elm)
        return results

    def get_pager(
        self, page_number: Optional[int] = 1, page_size: Optional[int] = 100
    ) -> Pager:
        assert page_size <= 1000
        factory = self.client.type_factory("ns0")
        return factory.Pager(pageSize=page_size, pageNumber=page_number)

    def _get_operation_list(
        self,
        date_from: datetime.datetime,
        date_to: datetime.datetime,
        page_size: Optional[int] = 1000,
        page_number: Optional[int] = 1,
    ) -> List[OrderedDict]:
        ftr = {"dateFrom": date_from, "dateTo": date_to}
        pager = self.get_pager(page_size=page_size, page_number=page_number)
        response = self.client.service.FindOperationsList(pager=pager, filter=ftr)
        result = zeep.helpers.serialize_object(response)
        return result.get("operation", [])

    def get_operations(
        self,
        date_from: datetime.datetime,
        date_to: datetime.datetime,
        page_size: Optional[int] = 1000,
        page_number: Optional[int] = 1,
    ) -> List[dict]:
        result = self._get_operation_list(
            date_from, date_to, page_size=page_size, page_number=page_number
        )
        return self._convert_to_list_of_dicts(result)

    def get_sum_per_period(
        self, date_from: datetime.datetime, date_to: datetime.datetime
    ) -> int:
        operations = self._get_operation_list(date_from, date_to)
        return (
            py_(operations)
            .map("attribute")
            .flatten()
            .reject(lambda elm: py_.get(elm, ["key"]) != "sourceamount")
            .map_("value")
            .map(int)
            .sum()
            .value()
        )

    def get_operation_sum(self, operation_id: int) -> int:
        # result = self.client.service.
        pass



import json
import logging
from typing import Optional

from django.views.generic import FormView, View, TemplateView
from django.http import HttpResponse, HttpResponseRedirect, JsonResponse

from moneta import signals
from moneta.models import PaymentSystem, Invoice
from moneta.configuration import moneta_config, UrlGenerator
from moneta.schemas import MonetaQueryParameters

log = logging.getLogger("moneta-log")


class SuccessView(TemplateView):

    def get(self, request, *args, **kwargs):
        return super().get(request, *args, **kwargs)


class FailView(TemplateView):

    def get(self, request, *args, **kwargs):
        return super().get(request, *args, **kwargs)


class CheckNotificationView(View):
    """Handler checking notification from payment server"""

    @staticmethod
    def create_invoice(params: MonetaQueryParameters) -> Invoice:
        values = params.dict()
        values["test_mode"] = params.get_test_mode()
        values["payment_system"] = params.get_payment_system()
        values["payment_system_unit_id"] = params.unit_id if params.unit_id else ""
        values["subscriber_id"] = params.subscriber_id if params.subscriber_id else ""
        del values["moneta_user"]
        del values["unit_id"]
        return Invoice.objects.create_check_invoice(
            transaction_id=params.transaction_id, values=values
        )

    def handle_input_data(self, input_data: dict) -> MonetaQueryParameters:
        params = MonetaQueryParameters(**input_data)
        invoice = self.create_invoice(params)
        signals.invoice_checking.send(sender=self, invoice=invoice)
        return params

    def post(self, request, *args, **kwargs):
        body = json.loads(request.body.decode("utf-8"))
        params = self.handle_input_data(input_data=body)
        return HttpResponse(params.xml_body(), content_type="text/xml")

    def get(self, request):
        params = self.handle_input_data(input_data=request.GET.dict())
        return HttpResponse(params.xml_body(), content_type="text/xml")


class PaidNotificationView(View):
    """Handler notification that user paid payment"""

    @staticmethod
    def finalize_invoice(params: MonetaQueryParameters) -> Optional[Invoice]:
        return Invoice.objects.finalize(transaction_id=params.transaction_id)

    def get(self, request):
        params = MonetaQueryParameters(**request.GET.dict())
        invoice = self.finalize_invoice(params)
        if invoice is None:
            return HttpResponse("FAIL")
        signals.invoice_paid.send(sender=self, invoice=invoice)
        return HttpResponse("SUCCESS")


class PaymentURLMixin:
    @staticmethod
    def init_url_generator() -> UrlGenerator:
        return moneta_config.url_generator()

    def get_payment_url(
        self, order_id: str, amount: int, payment_system: Optional[PaymentSystem] = None
    ) -> str:
        url_generator = self.init_url_generator()

        return url_generator.payment_url(
            payment_system=payment_system, order_id=order_id, amount=amount
        )


class PaymentInitApiView(View, PaymentURLMixin):
    """
    EXAMPLE View for RESTfull apis.
    It generates url for PayAnyWay payment system.
    Documentation: https://demo.moneta.ru/doc/MONETA.Assistant.ru.pdf
    """

    def get(self, request, order_id, amount):
        url = self.get_payment_url(order_id=order_id, amount=amount)
        return JsonResponse({"payment_url": url})


class PaymentInitFormView(FormView, PaymentURLMixin):
    """
    EXAMPLE: Standard Form View.
    It redirects user to PayAnyWay url payment system
    Documentation: https://demo.moneta.ru/doc/MONETA.Assistant.ru.pdf
    """

    def form_valid(self, form):
        url = self.get_payment_url(
            order_id=form.cleaned_data.get("transaction_id"),
            amount=form.cleaned_data.get("amount"),
            payment_system=form.cleaned_data.get("payment_system"),
        )
        return HttpResponseRedirect(url)
